// this class header file
#include "../include/Configuration.h"

#include <iostream>
#include <fstream>
#include <sstream>
#include <cstdlib>

using namespace std;

//---------------------------------------------------------------------- 
Configuration::Configuration()
{
    // the default randomseed will be generated from the time
  srand( time(0) % 900000000 );
  rseed = rand();
  
    // default: 1000 events
  nevents = 1000;
  
    // default Outputfilename
  OutputFileName = "output.root";
  
    // default: no Algorithm specified
  jet       = false;
  z         = false;
  wplusjet  = false;
  ztautau   = false;
  ttbar     = false;
  ue        = false;
  etmiss    = false;
  elasScat  = false;
  wtaunu    = false;
  bbbar     = false;
  user      = false;

    // default Jet parameter
  coneRadius          = 0.4;
  overlapThreshold    = 0.75;
  jet_ptmin           = 15.0;
  lepton_ptmin        = 15.0;
  DeltaR_lepton_track = 0.4;
  max_eta             = 2.5;
  min_pt              = 0.5;
  jetalgorithm        = 2;        // siscone = 1, antikt = 2
  
  FSR = true;
  ISR = true;
  MI  = true;
  
  IsOK = true;
}

//---------------------------------------------------------------------- 
Configuration::Configuration( const string &configfile )
{
  Configuration();
  IsOK = read( configfile );
}

//---------------------------------------------------------------------- 
Configuration::~Configuration()
{}

//---------------------------------------------------------------------- 
int Configuration::read( const string &configfile )
{
  ifstream myfile( configfile.c_str() );
  cout << "readConfigFile: Read in Configuration from file '" << configfile << "'" << endl;
  
  if ( !myfile.is_open() ) {
    cerr << "readConfigFile: Unable to open file" << endl;
    return 1;
  }

    // clean-up before	    
  ConfigFileNames.clear();
  InputFileNames.clear();

    // read line by line of the file
  string line;
  while ( !myfile.eof() ) {
    string parameter, str1;
    
    int     mylong  = -9999;
    double  myfloat = -9999.9;
    bool    mybool  = false;
    
    // write read lines into string line
    getline( myfile, line );
    
    // now split the line into the subwords
    istringstream instring( line );
    istringstream instring2( line );
    istringstream instring3( line );
    
    // and fill string or float
    instring >> parameter >> str1 ;
    instring2 >> parameter >> myfloat ;
    instring3 >> parameter >> mybool ;
    mylong = static_cast< long >( myfloat );
    
    // check for Comments. If the line starts with # than skip the line
    string::size_type pos = parameter.find( "#" );
    if ( pos != string::npos ) continue;
    
    // check for Comments. If the line starts with * than skip the line
    string::size_type pos2 = parameter.find( "*" );
    if ( pos2 != string::npos ) continue;
    
    // check for Comments. If the line starts with '/' than skip the line
    string::size_type pos3 = parameter.find( "/" );
    if( pos3 != string::npos ) continue;
    
    // skip if the string is empty
    if( parameter.empty() ) continue;
    
    // read pysub entries
    if ( ( parameter == "rseed" ) && ( mylong != -9999 ) ) {
      rseed = mylong;
      cout << "readConfigFile: Parameter " << parameter << "=" << rseed << endl;
    }
    else if ( ( parameter == "nevents" ) && ( mylong != -9999 ) ) {
      nevents = static_cast< unsigned int >( mylong );
      cout << "readConfigFile: Parameter " << parameter << "=" << nevents << endl;
    }
    else if ( ( parameter == "OutputFileName" ) && ( !str1.empty() ) ) {
      OutputFileName = str1;
      cout << "readConfigFile: Parameter " << parameter << "=" << OutputFileName << endl;
    }
    else if ( ( parameter == "ConfigFileNames" ) && ( !str1.empty() ) ) {
      ConfigFileNames.push_back( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << ConfigFileNames.back() << endl;
    }
    else if ( ( parameter == "InputFileNames" ) && ( !str1.empty() ) ) {
      InputFileNames.push_back( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << InputFileNames.back() << endl;
    }
    else if ( ( parameter == "jet_analysis" ) && ( !str1.empty() ) ) {
      jet = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << jet << endl;
    }
    else if ( ( parameter == "z_analysis" ) && ( !str1.empty() ) ) {
      z = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << z << endl;
    }
    else if ( ( parameter == "wplusjet_analysis" ) && ( !str1.empty() ) ) {
      wplusjet = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << wplusjet << endl;
    }
    else if ( ( parameter == "ztautau_analysis" ) && ( !str1.empty() ) ) {
      ztautau = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << ztautau << endl;
    }
    else if ( ( parameter == "ttbar_analysis" ) && ( !str1.empty() ) ) {
      ttbar = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << ttbar << endl;
    }
    else if ( ( parameter == "ue_analysis" ) && ( !str1.empty() ) ) {
      ue = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << ue << endl;
    }
    else if ( ( parameter == "etmiss_analysis" ) && ( !str1.empty() ) ) {
      etmiss = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << etmiss << endl;
    }
    else if ( ( parameter == "elasScat_analysis" ) && ( !str1.empty() ) ) {
      elasScat = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << elasScat << endl;
    }
    else if ( ( parameter == "wtaunu_analysis" ) && ( !str1.empty() ) ) {
      wtaunu = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << wtaunu << endl;
    }
    else if ( ( parameter == "bbbar_analysis" ) && ( !str1.empty() ) ) {
      bbbar = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << bbbar << endl;
    }
    else if ( ( parameter == "user_analysis") && ( !str1.empty() ) ) {
      user = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << user << endl;
    }
    else if ( ( parameter == "jet_coneRadius" ) && ( myfloat != -9999.9 ) ) {
      coneRadius = myfloat;
      cout << "readConfigFile: Parameter " << parameter << "=" << coneRadius << endl;
    }
    else if ( ( parameter == "jet_overlapThreshold" ) && ( myfloat != -9999.9 ) ) {
      overlapThreshold = myfloat;
      cout << "readConfigFile: Parameter " << parameter << "=" << overlapThreshold << endl;
    }
    else if ( ( parameter == "jet_ptmin" ) && ( myfloat != -9999.9 ) ) {
      jet_ptmin = myfloat;
      cout << "readConfigFile: Parameter " << parameter << "=" << jet_ptmin << endl;
    }
    else if ( ( parameter == "DeltaR_lepton_track" ) && ( myfloat != -9999.9 ) ) {
      DeltaR_lepton_track = myfloat;
      cout << "readConfigFile: Parameter " << parameter << "=" << DeltaR_lepton_track << endl;
    }
    else if ( ( parameter == "lepton_ptmin" ) && ( myfloat != -9999.9 ) ) {
      lepton_ptmin = myfloat;
      cout << "readConfigFile: Parameter " << parameter << "=" << lepton_ptmin << endl;
    }
    else if ( ( parameter == "max_eta" ) && ( myfloat != -9999.9 ) ) {
      max_eta = myfloat;
      cout << "readConfigFile: Parameter " << parameter << "=" << max_eta << endl;
    }
    else if ( ( parameter == "min_pt" ) && ( myfloat != -9999.9 ) ){
      min_pt = myfloat;
      cout << "readConfigFile: Parameter " << parameter << "=" << min_pt << endl;
    }
    else if ((parameter == "jetalgorithm") && (mylong!=-9999)){
      jetalgorithm=mylong;
      cout << "readConfigFile: Parameter " << parameter << "=" << jetalgorithm << endl;
    }
    else if ( ( parameter == "FSR" ) && ( !str1.empty() ) ) {
      FSR = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << FSR << endl;
    }
    else if ( ( parameter == "ISR" ) && ( !str1.empty() ) ) {
      ISR = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << ISR << endl;
    }
    else if ( ( parameter == "MI" ) && ( !str1.empty() ) ) {
      MI = isParamOn( str1 );
      cout << "readConfigFile: Parameter " << parameter << "=" << MI << endl;
    }
    else{
      cout << "readConfigFile: Parameter " << parameter << " not recognized!" << endl;
      return 1;
    }
  }
  myfile.close();
  
  return 0;
}

//---------------------------------------------------------------------- 
bool Configuration::isParamOn( const string &str ) const
{
  return ( str == "on" ) || ( str == "true" ) || ( str == "True" );
}

