/***********************************************************************
*                                                                      *
* File: MyCascadeWrapper.h                                             *
*                                                                      *
* written by Sebastian Johnert                                         *
* for the implementation of CASCADE                                    *
* into the HepMCAnalysis Tool                                          *
*                                                                      *
***********************************************************************/

//--------------------------------------------------------------------------
#ifndef MYCASCADE_WRAPPER_H
#define MYCASCADE_WRAPPER_H

//////////////////////////////////////////////////////////////////////////
// Wrapper for FORTRAN version of CASCADE
// This wrapper is NOT intended as a part of HepMC - it is only supplied
// for your convenience.
//////////////////////////////////////////////////////////////////////////
// 

#include <ctype.h>
#include <cstring>

//--------------------------------------------------------------------------
// CASCADE Common Block Declarations

extern "C" {
    extern struct {
      double pbeam[2][5];
      int kbeam[2][5], kint[2][5];
    } cabeam_;
}
#define cabeam cabeam_

extern "C" {
    extern struct {
      double avgi, sd;
      int nin, nout;
    } caeffic_;
}
#define caeffic caeffic_

extern "C" {
    extern struct {
      double thema, themi;
    } caelec_;
}
#define caelec caelec_

extern "C" {
    extern struct {
      int iglu;
    } cagluon_;
}
#define cagluon cagluon_

extern "C" {
    extern struct {
      int nia1, nia2, nir2, nf1, nf2;
    } cahard_;
}
#define cahard cahard_

extern "C" {
    extern struct {
      int ilha;
    } caherup_;
}
#define caherup caherup_

extern "C" {
    extern struct {
      int ihfla;
    } cahflav_;
}
#define cahflav cahflav_

extern "C" {
    extern struct {
      float plepin, ppin;
      int nfrag, ilepto, ifps, ihf, inter, isemih;
    } cainpu_;
}
#define cainpu cainpu_

extern "C" {
    extern struct {
      int ke, kp, keb, kph, kgl, kpa, nflav;
    } caluco_;
}
#define caluco caluco_

extern "C" {
    extern struct {
      int mult, ns_mult;
    } camult_;
}
#define camult camult_

extern "C" {
    extern struct {
      int ipro, iruna, iq2, irunaem;
    } capar1_;
}
#define capar1 capar1_

extern "C" {
    extern struct {
      int iwei;
      double alphs, pi, alph;
    } capar2_;
}
#define capar2 capar2_

extern "C" {
    extern struct {
      double am[18], pcm[4][18];
    } capar3_;
}
#define capar3 capar3_

extern "C" {
    extern struct {
      double q2, q2q;
    } capar4_;
}
#define capar4 capar4_

extern "C" {
    extern struct {
      double shat, ymax, ymin, q2max, q2min, xmax, xmin;
    } capar5_;
}
#define capar5 capar5_

extern "C" {
    extern struct {
      int lst[30], ires[2];
    } capar6_;
}
#define capar6 capar6_

extern "C" {
    extern struct {
      double sss, cm[4], dbcms[4];
    } caparton_;
}
#define caparton caparton_

extern "C" {
    extern struct {
      double pt2cut[20];
    } captcut_;
}
#define captcut captcut_

extern "C" {
    extern struct {
      double yy, yy_bar, xg, xg_bar, kt2_1, kt2_2, pt2h, shh;
    } caskin_;
}
#define caskin caskin_

//???, not completely sure
extern "C" {
    extern struct {
      char pdfpath[512];
    } caspdf_;
}
#define caspdf caspdf_

extern "C" {
    extern struct {
      double prkt1, prkt2, prktrem;
    } casprkt_;
}
#define casprkt casprkt_

extern "C" {
    extern struct {
      int iorder,itimshr,iccfm;
    } casshwr_;
}
#define casshwr casshwr_

extern "C" {
    extern struct {
      float acc1, acc2;
      int iint, ncb;
    } integr_;
}
#define integr integr_

extern "C" {
    extern struct {
      double scalfa;
    } scalf_;
}
#define scalf scalf_

extern "C" {
    extern struct {
      double qmi, ymi, qma, yma;
    } values_;
}
#define values values_

#endif  // MYCASCADE_WRAPPER_H
//--------------------------------------------------------------------------
