{
#include <stdio.h>
#include <fstream>
#include <vector>
#include <string>

gROOT->Reset();

std::vector< std::string > rootfiles;
std::string targetfilename="mergedFiles.root";

string line;
ifstream myfile ("mergeFiles.list");
if (myfile.is_open())
{
    // read line by line of the file                                                                                                                                                                         
    while (! myfile.eof() )
	{
	    // write read lines into string line                                                                                                                                                             
	    getline(myfile,line);
	    // check for Comments. If the line starts with # than skip the line                                                                                                                              
	    std::string::size_type pos = line.find("#");
	    if(pos != std::string::npos) continue;

	    // only root-files will be merged
	    std::string::size_type posExt = line.find(".root");
	    if(posExt == std::string::npos) continue;


	    cout << line << endl;

            // now split the line into the subwords
	    std::istringstream instring(line);
	    std::string string1;
	    std::string string2;

            instring >> string1 >> string2 ;

	    // check wether the file is a root-file
	    std::string::size_type posExt = string2.find(".root");
	    if(posExt == std::string::npos) continue;

	    // get files to be merged and targetfile
            if(string1 == "->" || string1 == "in")
                rootfiles.push_back(string2);
            else if(string1 == "<-" || string1 == "out")
		targetfilename=string2;
	    
	}
    myfile.close();
}
else {
cout << "Unable to open file";
gROOT->ProcessLine(".q");
}


TFile* targetfile=new TFile(targetfilename.c_str(),"RECREATE");
if(!targetfile->IsOpen()) {
    std::cout << targetfilename << " could not be opened! ... skip!" << std::endl;
    gROOT->ProcessLine(".q");
}

TFile* firstfile=new TFile(rootfiles[0].c_str());
if(!firstfile->IsOpen()) {
    std::cout << rootfiles[0] << " could not be opened! ... skip!" << std::endl;
    gROOT->ProcessLine(".q");
}

// need to scan rootfiles for subdirs and histograms
TKey *key_dir;
TKey *key_histo;

// get the iterator for the subdirectories in the file
TIter nextkey(firstfile->GetListOfKeys());

// loop over the TDirectories
while (key_dir = (TKey*)nextkey()) {
    TDirectory * currentrootdir = (TDirectory*)key_dir->ReadObj();
    if(!currentrootdir) continue;

    std::string classname= currentrootdir->ClassName();
    if(classname != std::string("TDirectoryFile")) continue;

    targetfile->mkdir(currentrootdir->GetName());

    // Loop over the histograms in the TDirectory
    TIter nextkey2(currentrootdir.GetListOfKeys());
    while (key_histo=(TKey*)nextkey2()){
	TH1D* currenthisto = (TH1D*)key_histo->ReadObj();
	if(!currenthisto) continue;

	std::string classname2= currenthisto->ClassName();
	if(classname2 != std::string("TH1D")) continue;

	std::string currenthistoname=currenthisto->GetName();
	// temporary rename histogram to avoit potential memory leak
	currenthisto->SetName("temporaryName");

	// Make a clone of the first histogram
	TH1D* clonehist = (TH1D*)currenthisto->Clone("clone");
	if(!clonehist) continue;

	// now loop over the files and add up histograms 
	for(unsigned int f=1 ; f < rootfiles.size(); f++){
	    TFile* currentfile=new TFile(rootfiles[f].c_str());

	    if(!firstfile->IsOpen()) {
		std::cout << rootfiles[0] << " could not be opened! ... skip this!" << std::endl;
		continue;
	    }

	    TDirectory * tempdir = (TDirectory*)currentfile->Get(currentrootdir->GetName());
	    if(!tempdir) continue;

	    // get next histogram from file or directory in file
	    TH1D* temphisto = (TH1D*)tempdir->Get(currenthistoname.c_str());
	    if(!temphisto) continue;

	    // add histogram
	    clonehist->Add(temphisto);

	    delete temphisto;
	    currentfile->Close();
	    delete currentfile; 
	}
	// change directory, rename histogram again to its original name and write histogram into file
	targetfile->cd(currentrootdir->GetName());
	clonehist->SetName(currenthistoname.c_str());
	clonehist->Write();

	delete currenthisto;
	delete clonehist;
    }
    delete currentrootdir;
}

targetfile->Close();
firstfile->Close();

}

