// ---------------------------------------------------------------------- 
// Enforce ATLAS style for histograms 
// Date: 23.2.2011
// ---------------------------------------------------------------------- 
void useAtlasStyle( double yTitleOffset = 1.0, bool atlasMarker = true, double leftMargin = 0.14 ) 
{
  using namespace std;
  cout << endl << "Setting ATLAS style..." << endl;

    // based on a style file from BaBar
  TStyle *atlasStyle= new TStyle( "ATLAS", "Atlas style" );

    // use plain black on white colors
  Int_t icol=0;
  atlasStyle->SetFrameBorderMode( icol );
  atlasStyle->SetCanvasBorderMode( icol );
  atlasStyle->SetPadBorderMode( icol );
  atlasStyle->SetPadColor( icol );
  atlasStyle->SetCanvasColor( icol );
  atlasStyle->SetStatColor( icol );
  atlasStyle->SetTitleFillColor( icol );
  //atlasStyle->SetFillColor(icol);

    // set the paper & margin sizes
  atlasStyle->SetPaperSize( 20, 26 );
  atlasStyle->SetPadTopMargin( 0.1 );
  atlasStyle->SetPadRightMargin( 0.05 );
  atlasStyle->SetPadBottomMargin( 0.16 );
  atlasStyle->SetPadLeftMargin( 0.12 );

    // use large fonts
  Int_t font=42;
  atlasStyle->SetTextFont( font );

  Double_t tsize=0.05;
  atlasStyle->SetTextSize( tsize );
  atlasStyle->SetLabelFont( font, "x" );
  atlasStyle->SetTitleFont( font, "x" );
  atlasStyle->SetLabelFont( font, "y" );
  atlasStyle->SetTitleFont( font, "y" );
  atlasStyle->SetLabelFont( font, "z" );
  atlasStyle->SetTitleFont( font, "z" );

  atlasStyle->SetLabelSize( tsize, "x" );
  atlasStyle->SetTitleSize( tsize, "x" );
  atlasStyle->SetLabelSize( tsize, "y" );
  atlasStyle->SetTitleSize( tsize, "y" );
  atlasStyle->SetLabelSize( tsize, "z" );
  atlasStyle->SetTitleSize( tsize, "z" );

  atlasStyle->SetTitleBorderSize( 0 );

    // use bold lines and markers
  if ( atlasMarker ) {
    atlasStyle->SetMarkerStyle( 20 );
    atlasStyle->SetMarkerSize( 1.2 );
  }
 
  atlasStyle->SetHistLineWidth( 3.0 );
  atlasStyle->SetLineStyleString( 2, "[12 12]" );     // postscript dashes

  atlasStyle->SetOptStat( 0 );
  atlasStyle->SetOptFit( 0 );

    // put tick marks on top and RHS of plots
  atlasStyle->SetPadTickX( 1 );
  atlasStyle->SetPadTickY( 1 );

    // DLA overrides
  atlasStyle->SetPadLeftMargin( leftMargin );
  atlasStyle->SetPadBottomMargin( 0.13 );
  atlasStyle->SetTitleYOffset( yTitleOffset );
  atlasStyle->SetTitleXOffset( 1.0 );

    // Enforce the style.
  gROOT->SetStyle( "ATLAS" );
  gROOT->ForceStyle();
  cout << "=> Done!" << endl;
}

