#!/usr/bin/python

import os
import sys
import shutil
import optparse

# private packages
import pages 

debugLevel = 2
#masterSettingsName = 'masterSettings.txt'

# ----------------------------------------------------------------------
def imageFilter( fileName ):
  # list of image sufficies
  plotsuff = [ 'gif', 'jpg', 'png', 'tiff', 'eps' ]
  
  suff = fileName.split( '.' )[-1]
  if suff in plotsuff:
    return 1 
  else: 
    return 0  
 
# ----------------------------------------------------------------------
def gifImageFilter( fileName ):
  plotsuff = [ 'gif' ]
  
  suff = fileName.split( '.' )[-1]
  if suff in plotsuff:
    return 1 
  else: 
    return 0  
 
# ----------------------------------------------------------------------
def removeDuplicates( fileNames ):
  tmplist = []
  for file1 in fileNames:
    alreadyExist = 0
    for file2 in tmplist:
      if file1.split( '.' )[0] == file2.split( '.' )[0]:
        alreadyExist = 1
        break
    if alreadyExist == 0:
      tmplist.append( file1 )
  return tmplist

# ---------------------------------------------------------------------- 
def debug_msg( message, level ):
  if debugLevel > level:
    print message

# ---------------------------------------------------------------------- 
# Analyses, processes and generators 
# ---------------------------------------------------------------------- 
analyses  = [ 'Analysis_DiJet', 'Analysis_Tau', 'Analysis_Top', 'Analysis_UE', 'Analysis_WplusJet', \
              'Analysis_Z', 'Analysis_EtMiss', 'Analysis_ElasScat']
processes = [ 'Process_DiJet', 'Process_Tau', 'Process_Top', 'Process_UE', 'Process_WplusJet', 'Process_ZplusJet', \
              'Process_singleW', 'Process_singleZ', 'Process_ElasScat']
generators = ['Pythia6', 'Pythia8', 'Herwigpp', 'fHerwig', 'Cascade']

# (1) Analysis <-> Analysis' source code mapping
srcCodeList = { 'Analysis_DiJet':           '../../Doxygen/html/DiJetAnalysis_8cc-source.html',
                'Analysis_Tau':             '../../Doxygen/html/TauAnalysis_8cc-source.html',
                'Analysis_Top':             '../../Doxygen/html/TopAnalysis_8cc-source.html',
                'Analysis_UE':              '../../Doxygen/html/UEAnalysis_8cc-source.html',
                'Analysis_WplusJet':        '../../Doxygen/html/WplusJetAnalysis_8cc-source.html',
                'Analysis_Z':               '../../Doxygen/html/ZAnalysis_8cc-source.html',
                'Analysis_EtMiss':          '../../Doxygen/html/EtMissAnalysis_8cc-source.html',
                'Analysis_ElasScat':        '../../Doxygen/html/ElasScatAnalysis_8cc-source.html' }

# (2) Process <-> Process steering mapping (depends on generator)
processSteerList = { 'Process_DiJet': {
                        'Pythia6':          '../../Doxygen/html/Pythia6__diJet_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__diJet_8cmnd-source.html',
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__dijet_8config-source.html',
                        'fHerwig':          '../../Doxygen/html/Herwig__dijet_8config-source.html',
                        'Cascade':          '../../Doxygen/html/Cascade__dijet_8config-source.html'
                    },
                    'Process_Tau': {
                        'Pythia6':          '../../Doxygen/html/Pythia6__tau_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__tau_8cmnd-source.html',
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__tau_8config-source.html',
                        'fHerwig':          '../../Doxygen/html/Herwig__tau_8config-source.html',
                        'Cascade':          '../../Doxygen/html/Cascade__tau_8config-source.html'
                    },
                    'Process_Top': {
                        'Pythia6':          '../../Doxygen/html/Pythia6__top_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__top_8cmnd-source.html',
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__top_8config-source.html',
                        'fHerwig':          '../../Doxygen/html/Herwig__top_8config-source.html',
                        'Cascade':          '../../Doxygen/html/Cascade__Top_8config-source.html'
                    },
                    'Process_UE': {
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__UE_8config-source.html'
                    },
                    'Process_WplusJet': {
                        'Pythia6':          '../../Doxygen/html/Pythia6__WplusJet_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__WplusJet_8cmnd-source.html',
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__WplusJet_8config-source.html',
                        'fHerwig':          '../../Doxygen/html/Herwig__WplusJet_8config-source.html',
                        'Cascade':          '../../Doxygen/html/Cascade__WplusJet_8config-source.html'
                    },
                    'Process_ZplusJet': {
                        'Pythia6':          '../../Doxygen/html/Pythia6__ZplusJet_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__ZplusJet_8cmnd-source.html',
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__ZplusJet_8config-source.html',
                        'fHerwig':          '../../Doxygen/html/Herwig__ZplusJet_8config-source.html',
                        'Cascade':          '../../Doxygen/html/Cascade__ZplusJet_8config-source.html'
                    },
                    'Process_singleW': {
                        'Pythia6':          '../../Doxygen/html/Pythia6__singleW_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__singleW_8cmnd-source.html',
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__singleW_8config-source.html',
                        'fHerwig':          '../../Doxygen/html/Herwig__singleW_8config-source.html',
                        'Cascade':          '../../Doxygen/html/Cascade__singleW_8config-source.html'
                    },
                    'Process_singleZ': {
                        'Pythia6':          '../../Doxygen/html/Pythia6__singleZ_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__singleZ_8cmnd-source.html',
                        'Herwigpp':         '../../Doxygen/html/Herwigpp__singleZ_8config-source.html',
                        'fHerwig':          '../../Doxygen/html/Herwig__singleZ_8config-source.html',
                        'Cascade':          '../../Doxygen/html/Cascade__singleZ_8config-source.html'
                    },
                    'Process_ElasScat': {
                        #'Pythia6':          '../../Doxygen/html/Pythia6__singleZ_8config-source.html',
                        'Pythia8':          '../../Doxygen/html/Pythia8__ElasScat_8cmnd-source.html'
                        #'Herwigpp':         '../../Doxygen/html/Herwigpp__singleZ_8config-source.html',
                        #'fHerwig':          '../../Doxygen/html/Herwig__singleZ_8config-source.html',
                        #'Cascade':          '../../Doxygen/html/Cascade__singleZ_8config-source.html',
                    }
    }

# (3) .... TODO 
processConfigList = { 'Process_DiJet': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_DiJet.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_DiJet.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_DiJet.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_DiJet.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_DiJet.config'
                    },
                    'Process_Tau': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_Tau.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_Tau.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_Tau.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_Tau.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_Tau.config'
                    },
                    'Process_Top': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_Top.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_Top.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_Top.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_Top.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_Top.config'
                    },
                    'Process_UE': {
                        #'Pythia6':          '',
                        #'Pythia8':          '',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_UE.config'
                        #'fHerwig':          '',
                        #'Cascade':          '',
                    },
                    'Process_WplusJet': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_WplusJet.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_WplusJet.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_WplusJet.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_WplusJet.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_WplusJet.config'
                    },
                    'Process_ZplusJet': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_ZplusJet.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_ZplusJet.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_ZplusJet.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_ZplusJet.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_ZplusJet.config'
                    },
                    'Process_singleW': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_singleW.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_singleW.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_singleW.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_singleW.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_singleW.config'
                    },
                    'Process_singleZ': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_singleZ.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_singleZ.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_singleZ.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_singleZ.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_singleZ.config'
                    },
                    'Process_ElasScat': {
                        'Pythia6':          '../../HepMC_Config_Scripts/Pythia6Process_ElasScat.config',
                        'Pythia8':          '../../HepMC_Config_Scripts/Pythia8Process_ElasScat.config',
                        'Herwigpp':         '../../HepMC_Config_Scripts/HerwigppProcess_ElasScat.config',
                        'fHerwig':          '../../HepMC_Config_Scripts/fHerwigProcess_ElasScat.config',
                        'Cascade':          '../../HepMC_Config_Scripts/CascadeProcess_ElasScat.config'
                    }
    }


# ---------------------------------------------------------------------- 
# Search for name of the process, analysis and generator...
# ---------------------------------------------------------------------- 
def extractAnalysisInfo( scriptPath ):
  processName = 'None'  
  analysisName = 'None'  
  generatorName = 'None'

  for p in processes:
    if scriptPath.find( p ) != -1:
      processName = p
      break
  for a in analyses:
    if scriptPath.find( a ) != -1:
      analysisName = a
      break
  for g in generators:
    if scriptPath.find( g ) != -1:
      generatorName = g
      break
  return ( processName, analysisName, generatorName )

# ---------------------------------------------------------------------- 
#                        execution starts here
# ---------------------------------------------------------------------- 
# The website will consist of the following parts:
#   - index.html - the main page
#   - overview.html - miniatures of plots
#   - empty.html - displayed when no histogram is selected
#   - analysis.html
# ---------------------------------------------------------------------- 
if __name__ == '__main__':
  # list image files in the current directory
  fullScriptName = os.path.abspath( sys.argv[ 0 ] )
  if debugLevel > 1: print '- full script name:', fullScriptName

  scriptPath = fullScriptName.split( '/mk_html.py' )[ 0 ]
  allFiles = os.listdir( scriptPath )
  
  currentDir = os.getcwd()
  if debugLevel > 1: print '- current directory:', currentDir
  if debugLevel > 1: print '- destination directory:', scriptPath 

  imgFiles = filter( imageFilter, allFiles )
  if debugLevel > 1: print '- found image files:', imgFiles
  
  #imgFiles = removeDuplicates( imgFiles )
  #if debugLevel > 1: print '- after duplicates removal:', imgFiles

  # index.html
  pages.createIndexPage( currentDir, scriptPath )
  
  # empty.html
  pages.createEmptyPage( currentDir, scriptPath )

  # overview.html
  gifImages = filter( gifImageFilter, imgFiles )
  pages.createOverviewPage( currentDir, scriptPath, gifImages )

    # extract process, analysis and generator names
  ( processName, analysisName, generatorName ) = extractAnalysisInfo( scriptPath )
  print 'Process:', processName, ', analysis:', analysisName, ', generator:', generatorName

    # temp
  #processName   = 'Process_Top'
  #analysisName  = 'Analysis_DiJet'
  #generatorName = 'Pythia6'

  configuration = processConfigList[ processName ][ generatorName ] 
  steering      = processSteerList[ processName ][ generatorName ] 
  sourceCode    = srcCodeList[ analysisName ]

  # analysis.html
  pages.createAnalysisPage( currentDir, scriptPath, configuration, steering, sourceCode )
  #pages.createAnalysisPage( currentDir, scriptPath, processName, analysisName, generatorName )

  # display area
  #pages.createDisplayPage( currentDir, scriptPath )

  # copy CSS style
  #shutil.copyfile( currentDir + "/templates/page_style.css", scriptPath + '/page_style.css' )

  print '> HTML page created successfully!'

