/***********************************************************************
*                                                                      *
* File: cascade.cc                                                     *
* This is a simple test program.                                       *
* based on the example cascade program                                 *
* from the MC School at DESY in April 2008                             *
*                                                                      *
* Modified by sj                                                       *
*                                                                      *
***********************************************************************/

//////////////////////////////////////////////////////////////////////////
// Events are read into the HepMC event record from the FORTRAN HEPEVT 
// common block using the IO_HEPEVT strategy -- nothing is done with them.
// This program is just used to find the total time required to transfer
// from HEPEVT into the HepMC event record.
//////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "HepMC/PythiaWrapper.h"
#include "CascadeWrapper.h"
#include "MyCascadeWrapper.h"
#include "HepMC/IO_HEPEVT.h"
//#include "HepMC/IO_Ascii.h"
#include "HepMC/IO_GenEvent.h"
#include "HepMC/GenEvent.h"
#include "PythiaHelper.h"

#include <stdio.h>
#include "HepMC/IO_AsciiParticles.h"
#include "HepMC/SimpleVector.h"

#include "TH1.h"
#include "TFile.h"
#include "TMath.h"
#include <TCanvas.h>

#include "TF1.h"
#include <TH2.h>
#include <TStyle.h>
#include "TLegend.h"
#include <TTree.h>
#include <TString.h>

// include our own classes
#include "Configuration.h"
#include "readCascadeConfigFile.h"

extern "C" {
  extern struct {
    int Nevent;
  } steer1_;
}
#define steer1 steer1_

using namespace std;

// ---------------------------------------------------------------------- 
int main( int argc, const char* argv[] ) 
{ 
  if(argc <= 2 ) { //ap
    std::cout << "Usage: " << argv[0] << " <Configuration File>" << " <name of file to generate to>"<< std::endl; //ap
    return 0;
  }

  Configuration* currentconfig = new Configuration();
  if ( currentconfig->read( argv[1] ) ) {
    cout << " ... skip !" << endl;
    return 0;
  }	

  
  //
  //........................................HEPEVT
  // CASCADE (via PYTHIA6) uses HEPEVT with 10000 entries and 
  // 8-byte floating point numbers. We need to explicitly pass
  // this information to the HEPEVT_Wrapper.
  //
  //    HepMC::HEPEVT_Wrapper::set_sizeof_int(4);
  //    HepMC::HEPEVT_Wrapper::set_sizeof_real(8);
  //    HepMC::HEPEVT_Wrapper::set_max_number_entries(10000);
  //
  pyjets.n=0;     // ensure dummyness of the next call
  call_pyhepc(1); // mstu(8) is set to NMXHEP in this dummy call (version >=6.404)
  //HepMC::HEPEVT_Wrapper::set_max_number_entries(pydat1.mstu[8-1]);
  HepMC::HEPEVT_Wrapper::set_max_number_entries(10000);
  HepMC::HEPEVT_Wrapper::set_sizeof_real(8);
  
  //	
  //    initPythia();
  //........................................CASCADE INITIALIZATIONS
  //  call_rluxgo(lux,iseed,k1,k2)
  // lux = luxory level
  // iseed = seed for random number generator
  //    call_rluxgo(4,87777886,0,0);
  //--initialise CASCADE parameters

  call_casini();

  //call steering
  //call_steer();

  //-- change standard parameters of CASCADE
  call_cascha();
  
  //change cascade parameters
  vector< string > cascade_commands;
  // read in commandlines
  for (unsigned int files=0; files<(currentconfig->ConfigFileNames).size(); files++)
    readConfigFile((currentconfig->ConfigFileNames)[files]);

  //-- change standard parameters of JETSET/PYTHIA
  call_pytcha();

  //-- set up for running CASCADE  
  call_cascade();

  //-- print result from integration
  call_caend(1);   

  //
  //........................................HepMC INITIALIZATIONS
  //
  // Instantiate an IO strategy for reading from HEPEVT.
  HepMC::IO_HEPEVT hepevtio;
  //
  HepMC::IO_GenEvent ascii_out(argv[2],std::ios::out);  //add ap
  // Instantial an IO strategy to write the data to file - it uses the 
  //  same ParticleDataTable
  //HepMC::IO_GenEvent ascii_io("example_MyCASCADE.dat",ios::out);
  //
  //........................................EVENT LOOP
  //int Nevent = steer1.Nevent;
  int Nevent = currentconfig->nevents;

  for ( int iEvent = 1; iEvent <= Nevent; iEvent++ ) {
    if ( iEvent == 1 || iEvent % 100 == 0 ) {
      cout << "Processing Event Number "  << iEvent << endl;
    }
    call_event();      // generate one event with CASCADE

    // pythia pyhepc routine convert common PYJETS in common HEPEVT
    call_pyhepc( 1 );
    HepMC::GenEvent* hepmcevt = hepevtio.read_next_event();
    //
    // add some information to the event
    hepmcevt->set_event_number(iEvent);
    hepmcevt->set_signal_process_id(11);


    ascii_out << hepmcevt;    //add ap
    // we also need to delete the created event from memory
    delete hepmcevt;

  }
  //........................................TERMINATION
  //  Print out of generated event summary
  call_caend(2);
  // write out some information from Pythia to the screen
  //    call_pystat( 1 );    
  

  // clean up config object
  delete currentconfig;

  cout << "cascade: program ends" << endl;

  cout << "Run successfully!" << endl;
  return 0;
}

