/***********************************************************************
*                                                                      *
* File: pythia6_jet.cc                                                 *
* This is a simple test program.                                       *
* Copyright \ufffd 2005 Torbj\ufffdrn Sj\ufffdstrand                   *
*                                                                      *
* Modified by sj and ca                                                *
*                                                                      *
***********************************************************************/
//#include "Pythia.h"

//#include "LHAFortran.h"

//#include "HepMCInterface.h"

// this is needed to generate some random numbers
#include <stdlib.h>
#include <time.h>

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include "HepMC/PythiaWrapper.h"
#include "HepMC/IO_HEPEVT.h"
// #include "HepMC/IO_Ascii.h"
#include "HepMC/IO_BaseClass.h"
#include "HepMC/GenEvent.h"
#include "HepMC/SimpleVector.h"
#include "MyPythia6Wrapper.h"

#include "HepMC/IO_GenEvent.h"  //ap add

// include our own classes
#include "Configuration.h"
#include "readpythiaConfigFile.h"

using namespace std;

// ---------------------------------------------------------------------- 
int main(int argc, const char* argv[]) 
{
  if(argc <= 2 ) { //ap
    std::cout << "Usage: " << argv[0] << " <Configuration File>" << " <name of file to generate to>"<< std::endl; //ap
    return 0;
  }
  
  
  Configuration* currentconfig = new Configuration();
  if(currentconfig->read(argv[1]))
    {
      cout << " ... skip !" << endl;
      return 0;
    }
  


  //........................................HEPEVT
  //  Standard Pythia >6.1 uses HEPEVT with NMXHEP=4000 entries and 8-byte
  //  floating point numbers. We need to explicitly pass this information
  //  to the HEPEVT_Wrapper.
  pyjets.n=0; // ensure dummyness of the next call
  call_pyhepc(1); // mstu(8) is set to NMXHEP in this dummy call (version >=6.404)
  //HepMC::HEPEVT_Wrapper::set_max_number_entries(pydat1.mstu[8-1]);
  HepMC::HEPEVT_Wrapper::set_max_number_entries(10000);
  HepMC::HEPEVT_Wrapper::set_sizeof_real(8);
  
  //
  //........................................PYTHIA INITIALIZATIONS
  // (Some platforms may require the initialization of pythia PYDATA block
  //  data as external - if you get pythia initialization errors try
  //  commenting in/out the below call to initpydata() )
  // initpydata();
  //
  
  // set random number seed (mandatory!) ... this may be overwritten by function readConfigFile
  pydatr.mrpy[1-1] = currentconfig->rseed ;
  
  //int tunemode=300;
  //pytune( &tunemode );
  
  cout << "Configuration Parameter List:" << endl;
  
  for (unsigned int files=0; files<(currentconfig->ConfigFileNames).size(); files++)
    readConfigFile((currentconfig->ConfigFileNames)[files]);

  // these three lines are set in the config file Pythia6_Common.config
  //   // master switches for FSR, ISR and MI
  //   pypars.mstp[61-1]=int(currentconfig->ISR); 
  //   pypars.mstp[71-1]=int(currentconfig->FSR); 
  //   pypars.mstp[81-1]=int(currentconfig->MI); 

  // Call pythia initialization
  call_pyinit("CMS", "p", "p", 14000.);
  
  //........................................HepMC INITIALIZATIONS
  //
  // Instantiate an IO strategy for reading from HEPEVT.
  HepMC::IO_HEPEVT hepevtio;
  //
  // Instantiate an IO strategy to write the data to file - it uses the 
  //  same ParticleDataTable
  //HepMC::IO_Ascii ascii_io("test_pythia_hepmc.dat",ios::out);
  
  //.......EVENT LOOP:
  
  int nShow = 50;
  int nShowPace = max(1,int(currentconfig->nevents)/nShow);
  
  HepMC::IO_GenEvent ascii_out(argv[2],std::ios::out);  //ap add
  
  for (unsigned int iEvent = 1 ; iEvent <= currentconfig->nevents; iEvent++) {
    
    // Begin event loop. 
    if (iEvent%nShowPace == 0) cout << "pythia6:  Now begin event "
				    << iEvent << endl;
    
    call_pyevnt();      // generate one event with Pythia
    // pythia pyhepc routine converts common PYJETS in common HEPEVT
    call_pyhepc(1);
    HepMC::GenEvent* evt = hepevtio.read_next_event();
    // add some information to the event
    evt->set_event_number(iEvent);
    evt->set_signal_process_id(20);
    // write the event out to the ascii file
    //    ascii_io << evt;
    
    
    // OUTPUT
    ascii_out << evt;   //ap add 

    // we also need to delete the created event from memory
    delete evt;
  } // end of event loop
  
  
  // write out some information from Pythia to the screen
  call_pystat(1);    

  // clean up config object
  delete currentconfig;
  
  cout<<"pythia6:  program ends"<<endl;

  cout<<"Run successfully!"<<endl;
  return 0;
}

