/*Generator dependant!!!*/

/***********************************************************************
*                                                                      *
* File: pythia8_jet.cc                                                 *
* This is a simple test program.                                       *
* Copyright \ufffd 2005 Torbj\ufffdrn Sj\ufffdstrand                   *
*                                                                      *
* Modified by sj and ca                                                *
*                                                                      *
***********************************************************************/
#include "Pythia.h"

#include "LHAFortran.h"

#include "HepMCInterface.h"

#include "HepMC/GenEvent.h"

// this is needed to generate some random numbers                                                                                                                                                                    
#include <stdlib.h>
#include <time.h>

#include <iostream>
#include <stdio.h>
#include "HepMC/IO_GenEvent.h"
#include "HepMC/GenEvent.h"
#include "HepMC/IO_AsciiParticles.h"
#include "HepMC/SimpleVector.h"

using namespace Pythia8; 
using namespace std;

#include "Configuration.h"


/**************************************************************************/

int main(int argc, const char* argv[]) {
  
  if(argc <= 2 ) { //ap
    std::cout << "Usage: " << argv[0] << " <Configuration File>" << " <name of file to generate to>"<< std::endl; //ap
    return 0;
  }
  
  Configuration* currentconfig = new Configuration();
  if(currentconfig->read(argv[1]))
    {
      //std::cout << " ... skip !" << std::endl;
      return 0;
    }
  

  HepMC::I_Pythia8 ToHepMC;

  // Generator. Process selection. LHC initialization.
  //Pythia pythia;
  Pythia pythia(std::getenv("Pythia8DATA"));

  char processline[64];
  // Read in commands from external file.
  for (unsigned int files=0; files<currentconfig->ConfigFileNames.size(); files++)
      {
	  //std::cout << "pythia8: Will use settings from file " << currentconfig->ConfigFileNames[files].c_str() << std::endl;
	  pythia.readFile(currentconfig->ConfigFileNames[files].c_str());
      }

  // as done for the random seed, if it is specified
  pythia.readString("Random:setSeed = on");
  //sprintf(processline,"Random:seed = %ld",currentconfig->rseed);
  pythia.readString(processline);

  // master switches for FSR, ISR and MI
  //sprintf(processline,"PartonLevel:FSR = %d",int(currentconfig->FSR));
  pythia.readString(processline);

  //sprintf(processline,"PartonLevel:ISR = %d",int(currentconfig->ISR));
  pythia.readString(processline);

  //sprintf(processline,"PartonLevel:MI = %d",int(currentconfig->MI));
  pythia.readString(processline);

  // Settings can also be changed with this command
  //pythia.readString("WeakBosonAndParton:all = on");

  pythia.init( 2212, 2212, 14000.);

  // Shorthand for the event and for settings.
  Event& event = pythia.event;
  Settings& settings = pythia.settings;

  // Extract settings to be used in the main program.
  int nEvent = currentconfig->nevents;
  //sj//  int nEvent = settings.mode("Main:numberOfEvents");
  //int nList = settings.mode("Main:numberToList");
  int nShow = settings.mode("Main:timesToShow");
  int nAbort = settings.mode("Main:timesAllowErrors");
  bool showChangedSettings = settings.flag("Main:showChangedSettings");
  bool showAllSettings = settings.flag("Main:showAllSettings");
  bool showAllParticleData = settings.flag("Main:showAllParticleData");

  // List changed data.
  if (showChangedSettings) settings.listChanged();
  if (showAllSettings) settings.listAll();

  // List particle data.  
  //if (showAllParticleData) ParticleDataTable::listAll();
  //starting at least for Pythia8 135 use the following line
  if (showAllParticleData) pythia.particleData.listAll();

  // Begin event loop.
  int nShowPace = max(1,nEvent/nShow); 
  int iAbort = 0; 
  HepMC::IO_GenEvent ascii_out(argv[2],std::ios::out);  //add ap
  for (int iEvent = 0; iEvent < nEvent; ++iEvent) {

        if (iEvent%nShowPace == 0) 
	    //cout << "pythia8: Now begin event "  << iEvent << endl;

    // Generate events. Quit if too many failures.
    if (!pythia.next()) {
      if (++iAbort < nAbort) continue;
      //cout << "pythia8:  Event generation aborted prematurely, owing to error!\n"; 
      break;
    }
 
    // List first few events, both hard process and complete events.
    //if (iEvent < nList) { 
    //if (iEvent < nEvent) { 
    //  pythia.process.list();
      //event.list();
    // }

    //cout << "before ctor" << endl;
    HepMC::GenEvent* hepmcevt = new HepMC::GenEvent();
    //cout << "after ctor" << endl;
    ToHepMC.fill_next_event( event, hepmcevt );
    //cout << "fill nex" << endl;
    
    

    // OUTPUT
    ascii_out << hepmcevt;    //add ap
    

    delete hepmcevt;
    // End of event loop.
  }

  // gen>>>>>>>>>>>>>>>>>>>>>>>>
  // Final statistics.
  pythia.statistics();
  //  pythia.particleData.listAll();
  pythia.settings.writeFile("allSettings.cmnd",true);


  // clean up config object
  delete currentconfig;

  // Tidy up
  std::cout<<"pythia8: program ends"<<std::endl;
  std::cout<<"Run successfully!"<<std::endl;
  
  return 0;
}

