#ifndef _HEPMCANA_CONFIGURATION_H_
#define _HEPMCANA_CONFIGURATION_H_

#include <string>
#include <vector>

class baseAnalysis;

/**
@class Configuration.h
@brief Read the configuration into the executable for event generation  

@author Cano Ay Dec 2008 
*/
class Configuration
{
public:
  Configuration();
  Configuration(const std::string& configFile);
    
  virtual ~Configuration();

  bool read(const std::string& configFile);

  std::vector<baseAnalysis*> instantiateAnalyses() const;

public:
  long rseed;
  unsigned int nevents;
  
  std::string OutputFileName;
  std::vector<std::string> ConfigFileNames;
  std::vector<std::string> InputFileNames;

  // enable Algorithm    
  bool jet;
  bool z;
  bool wplusjet;
  bool ztautau;
  bool ttbar;
  bool ue;
  bool etmiss;
  bool elasScat;
  bool wtaunu;
  bool bbbar;
  bool user;

  // Jet specific Variables
  double coneRadius;
  double overlapThreshold;
  double jet_ptmin;
  double lepton_ptmin;
  double DeltaR_lepton_track;
  double max_eta;
  double min_pt;
  int    jetalgorithm;

  bool FSR;
  bool ISR;
  bool MI;

  bool IsOK;

private:
  inline bool isParamOn(const std::string &str) const;
  void useDefaultSettings();
};

#endif

