#ifndef ElASSCAT_ANALYSIS_H_
#define ElASSCAT_ANALYSIS_H_

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;

/**
 @class ElasScatAnalysis.h                                                  
 @brief This class is used for hepmc based validation of elastic scattered events
                                                                       
 Init() is used to initialise histograms                               
 Process() is used to process the events and do the analysis and       
 the ouput (histograms) will be saved in a root file in the ElasScat folder
                                                                       
 @author provided by Hasko Stenzel, September 2009  */

class ElasScatAnalysis: public baseAnalysis
{
public:
  ElasScatAnalysis();
  virtual ~ElasScatAnalysis();

  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);
  
private:
  TH1D *n_proton;
  TH1D *p1_eta;
  TH1D *p2_eta;
  TH1D *p1_pt;
  TH1D *p2_pt;
  TH1D *p1_theta;
  TH1D *p2_theta;
  TH1D *p1_phi;
  TH1D *p2_phi;
  TH1D *t_spect;
  TH1D *tau_spect;
  TH1D *pi1_en;
  TH1D *pi1_divx;
  TH1D *pi1_divy;
  TH1D *pi2_en;
  TH1D *pi2_divx;
  TH1D *pi2_divy;
};

#endif      // ElASSCAT_ANALYSIS_H_

