#ifndef JET_ANALYSIS_H_
#define JET_ANALYSIS_H_

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;

/**
@class JetAnalysis.h
@brief Class to analyse events of the type QCD 2->2 Processes. 
       Examples to generate these events are given in the specific generator 
       directory, e.g. examples/pythia6 ; the histogramm output is stored 
       in the Jet folder of the root file.
       

@author Cano Ay Dec 2008 */

class JetAnalysis: public baseAnalysis
{
public:
  JetAnalysis();
  virtual ~JetAnalysis();
  
  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);
  
private:
  TH1D *m_jet_count;
  TH1D *m_jet_pt;
  TH1D *m_jet_pt_log;
  TH1D *m_leadingjet_pt;
  TH1D *m_leadingjet_pt_log;
  TH1D *m_secondleadingjet_pt;
  TH1D *m_secondleadingjet_pt_log;
  //TH1D *m_jet_pt_high;
  //TH1D *m_jet_pt_high_log;
  TH1D *m_jet_phi;
  TH1D *m_jet_eta;
  
  TH1D *m_charged_particle_multiplicity;
  TH1D *m_charged_particle_mean_pt;
  TH1D *m_charged_particle_temp_pt;
  TH1D *m_charged_particle_pt;
  TH1D *m_charged_particle_pt_log;
  TH1D *m_charged_particle_rms_pt;
  TH1D *m_charged_particle_pdgID;

  TH1D *m_jet_Deta_leadingJet_secondJet;
  TH1D *m_jet_Dphi_leadingJet_secondJet;
  TH1D *m_jet_DR_leadingJet_secondJet;
};

#endif      // JET_ANALYSIS_H_

