#ifndef UE_ANALYSIS_H_
#define UE_ANALYSIS_H_

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;
class TFile;

/**
@class UEAnalysis.h
@brief Class to study underlying events. Currently the UE is studied for 
       events with at least one  jet, e.g. QCD 2->2, pp->W+jet, pp->Z+jet.
       Extension to DrellYan events will follow.
       Examples to generate e.g events are given in the specific generator
       directory, e.g. examples/pythia6 ; the histogramm output is stored
       in the UE folder of the root file.

@author Sebastian Johnert, Judith Katzy, Zhonghua Qin Dec 2008 */

class UEAnalysis: public baseAnalysis
{
public:
  UEAnalysis();
  virtual ~UEAnalysis();
  
  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);
  virtual int Finalize();
  virtual int Finalize(TFile* output);
  /** averagedHistograms: function to calculate averaged histograms for the UE analysis*/
  virtual std::vector<TH1D*> averagedHistograms();

private:
  TH1D *m_NchargedToward[26]; 
  TH1D *m_NchargedTransverse[26]; 
  TH1D *m_NchargedAway[26]; 
  
  TH1D *m_NchargedMeanToward; 
  TH1D *m_NchargedMeanTransverse; 
  TH1D *m_NchargedMeanAway; 

  TH1D *m_PtsumToward[26]; 
  TH1D *m_PtsumTransverse[26]; 
  TH1D *m_PtsumAway[26]; 
  
  TH1D *m_PtsumMeanToward; 
  TH1D *m_PtsumMeanTransverse; 
  TH1D *m_PtsumMeanAway; 

  TH1D *m_Njet; 
  TH1D *m_Ptjet;
  TH1D *m_Ptjet_log;
  TH1D *m_Ptleadingjet;
  TH1D *m_Ptleadingjet_log;

  TH1D *m_charged_particle_pt;

  /** definition for the variable bin size for the <Ncharged> vs pT calculation*/
  int m_nbin_pT;
  //size of m_nbinRange has to be nbin + 1 due to 0 at the beginning
  double m_nbinRange_pT[26];
};

#endif      // UE_ANALYSIS_H_

