#ifndef WPLUSJET_ANALYSIS_H_
#define WPLUSJET_ANALYSIS_H_

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;

/**
@class WplusJetAnalysis.h
@brief Class to analyse events of the type pp->W+jet Processes.
       Examples to generate these events are given in the specific generator
       directory, e.g. examples/pythia6 ; the histogramm output is stored
       in the WplusJet folder of the root file.

@author Cano Ay Dec 2008 */

class WplusJetAnalysis: public baseAnalysis
{
public:
  WplusJetAnalysis();
  virtual ~WplusJetAnalysis();
  
  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);
  
private:
  TH1D *m_evtnr;
  TH1D *m_charged_particle_multiplicity;
  TH1D *m_charged_particle_mean_pt;
  TH1D *m_charged_particle_temp_pt;
  TH1D *m_charged_particle_pt;
  TH1D *m_charged_particle_rms_pt;
  TH1D *m_charged_particle_pdgID;

  TH1D *m_jet_Deta_leadingJet_lepton;
  TH1D *m_jet_Dphi_leadingJet_lepton;
  TH1D *m_jet_DR_leadingJet_lepton;

  TH1D *m_jet_Deta_leadingJet_lepton_singlejet;
  TH1D *m_jet_Dphi_leadingJet_lepton_singlejet;
  TH1D *m_jet_DR_leadingJet_lepton_singlejet;

  TH1D *m_jet_Deta_leadingJet_lepton_multijet;
  TH1D *m_jet_Dphi_leadingJet_lepton_multijet;
  TH1D *m_jet_DR_leadingJet_lepton_multijet;

  TH1D *m_W_count;
  TH1D *m_W_pt;
  TH1D *m_W_pt_log;
  TH1D *m_W_pt_high;
  TH1D *m_W_pt_high_log;
  TH1D *m_W_eta;
  TH1D *m_W_rapidity;
  TH1D *m_W_phi;
  TH1D *m_W_trans_mass;
  TH1D *m_W_trans_mass_log;

  TH1D *m_jet_count;
  TH1D *m_jet_pt;
  TH1D *m_jet_pt_log;

  TH1D *m_lepton_count;
  TH1D *m_lepton_pt;
  TH1D *m_lepton_pt_log;
  TH1D *m_lepton_pt_high;
  TH1D *m_lepton_pt_high_log;
  TH1D *m_lepton_eta;
  TH1D *m_lepton_phi;
  TH1D *m_lepton_pdgid;

  double m_coneRadius;
  double m_overlapThreshold;
  double m_jet_ptmin;
  double m_DeltaR_lepton_track;
  double m_lepton_ptmin;
};

#endif    // WPLUSJET_ANALYSIS_H_

