#ifndef WtaunuAnalysis_H
#define WtaunuAnalysis_H

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;

/**
                                                                       
 @class WtaunuAnalysis.h
 @brief This class is used for hepmc based validation of Wtaunu events  

 Init() is used to initialise histograms                               
 Process() is used to process the events and do the analysis and       
                                                                       
 The ouput (histograms) will be saved in a root file                   
                                                                       
 @author Chris Malena Delitzsch, August 2010
 @author Sebastian Johnert, January 2010 (modification and implementation in the tool)
                                                                       
*/

class WtaunuAnalysis: public baseAnalysis
{
 public:

  WtaunuAnalysis();
  virtual ~WtaunuAnalysis();

  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);
  
 private:
  TH1D *m_evtnr;
  TH1D *m_h_njets;

  TH1D *m_W_count;
  TH1D *m_W_charge;
  TH1D *m_W_mass;
  TH1D *m_W_mt;  
  TH1D *m_W_trans_mass;
  TH1D *m_W_pt;
  TH1D *m_W_eta;
  TH1D *m_W_phi;
  TH1D *m_W_rapidity;

  TH1D *m_charged_particle;
  TH1D *m_charged_particle_pdgID;
  TH1D *m_ptstable_charged;
  TH1D *m_etastable_charged;
  TH1D *m_phistable_charged;

  TH1D *m_tau_mass;
  TH1D *m_tau_pt;
  TH1D *m_tau_eta;
  TH1D *m_tau_phi;
  TH1D *m_nTrack_tau;

  TH1D *m_Delta_Phi_TauNeutrino;
  
  TH1D *m_nu_pt;
  TH1D *m_nu_px;
  TH1D *m_nu_py;
  TH1D *m_nu_pz; 

  TH1D *m_zoom_nu_px1;
  TH1D *m_zoom_nu_py1;
  TH1D *m_zoom_nu_pz1;

};

#endif    // Wtaunu_Analysis_H_
