#ifndef _HEPMCANALYSIS_Z_H_
#define _HEPMCANALYSIS_Z_H_

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;

/**
@class ZAnalysis.h
@brief Class to analyse processes of the type QCD pp->Z->e+e-, Z->mu+mu-
       Examples to generate these events are given in the specific generator
       directory, e.g. examples/pythia6 ; the histogramm output is stored
       in the Zanalysis folder of the root file.

@author Cano Ay Dec 2008 */
class ZAnalysis: public baseAnalysis
{
public:
  enum EParticleType {
    Electron = 11, Muon = 13, Tau = 15 
  };

  ZAnalysis();
  virtual ~ZAnalysis();

  virtual int Init( double tr_max_eta, double tr_min_pt );
  virtual int Process( HepMC::GenEvent *hepmcevt );

private:
  bool FSVector( HepMC::GenEvent::particle_const_iterator &p, EParticleType productPid, HepMC::FourVector &fsVector );
  void FillZHistograms( const HepMC::FourVector &fsVec );

  TH1D *m_evtnr;
  TH1D *m_charged_particle_multiplicity;
  TH1D *m_charged_particle_pt;
  TH1D *m_charged_particle_mean_pt;
  TH1D *m_charged_particle_temp_pt;
  TH1D *m_charged_particle_rms_pt;
  TH1D *m_charged_particle_pdgID;

  TH1D *m_Z_pt;
  TH1D *m_Z_pt_log;
  TH1D *m_Z_pt_high;
  TH1D *m_Z_pt_high_log;
  TH1D *m_Z_eta;
  TH1D *m_Z_phi;
  TH1D *m_Z_mass;
  TH1D *m_Z_mass_log;

  TH1D *m_Z_pt_prop;
  TH1D *m_Z_pt_prop_log;
  TH1D *m_Z_pt_high_prop;
  TH1D *m_Z_pt_high_prop_log;
  TH1D *m_Z_eta_prop;
  TH1D *m_Z_phi_prop;
  TH1D *m_Z_mass_prop;
  TH1D *m_Z_mass_prop_log;

  TH1D *m_jet_count;
  TH1D *m_jet_pt;
  TH1D *m_jet_pt_log;
};

#endif 

