#ifndef ZTAUTAU_ANALYSIS_H_
#define ZTAUTAU_ANALYSIS_H_

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;

/**
 @class ZtautauAnalysis.h                                                  
 @brief This class is used for hepmc based validation of pp->Z->tautau events
                                                                       
 Init() is used to initialise histograms                               
 Process() is used to process the events and do the analysis and       
 The ouput (histograms) will be saved in a root file in the Ztautau folder
                                                                       
 @author Sebastian Johnert, October 2008  */

class ZtautauAnalysis: public baseAnalysis
{
public:
  ZtautauAnalysis();
  virtual ~ZtautauAnalysis();
  
  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);
  
private:
  TH1D *m_evtnr;
  TH1D *m_taupt;
  TH1D *m_taupt_log;
  TH1D *m_taueta;
  TH1D *m_tauphi;
  TH1D *m_ptstable;
  TH1D *m_ptstable_log;
  TH1D *m_etastable;
  TH1D *m_phistable;
  TH1D *m_E_stable;
  TH1D *m_E_stable_log;
  TH1D *m_ptstable_charged;
  TH1D *m_ptstable_charged_log;
  TH1D *m_etastable_charged;
  TH1D *m_phistable_charged;
  TH1D *m_E_stable_charged;
  TH1D *m_E_stable_charged_log;
  TH1D *m_cmultpart;
  TH1D *m_nTrack_tau;
  TH1D *m_pt_hightrack;
  TH1D *m_pt_hightrack_log;

  TH1D *m_jet_count;
  TH1D *m_jet_pt;
  TH1D *m_jet_pt_log;
};

#endif    // ZTAUTAU_ANALYSIS_H_

