#ifndef bbbarAnalysis_H
#define bbbarAnalysis_H

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}

class TH1D;
class TFile;

/**
                                                                       
 @class bbbarAnalysis.h
 @brief This class is used for hepmc based validation of bbbar events  

 Init() is used to initialise histograms                               
 Process() is used to process the events and do the analysis and       
                                                                       
 The ouput (histograms) will be saved in a root file                   
                                                                       
                                                                       
*/

class bbbarAnalysis : public baseAnalysis {

public:

  bbbarAnalysis();
  bbbarAnalysis(char *);

  void getBbbar(HepMC::GenEvent *evt);

  int getBcount() {return Bcount;}
  int getBcountTotal() {return BcountTotal;}
  int getBdcount() {return Bdcount;}
  int getBucount() {return Bucount;}
  int getBscount() {return Bscount;}
  int getBarions() {return Barions;}
  int getChargedPartMB() {return ChargedPartMB;}
  int getChargedPartBBbar() {return ChargedPartBBbar;}

  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);

  virtual ~bbbarAnalysis();

private:

  void clearCounters();
  void initPlots();

  TFile *hOutputFile;
  char* OutputFileName;   // ???

  TH1D *hPtmb;
  TH1D *hPtbbbar;
  TH1D *hPtbhadr;
  TH1D *hEtabhadr;

  // the number of 'b events'
  int Bcount;
  int BcountTotal;
  // the number of Bd/Bu/Bs/b-baryons
  int Bdcount;
  int Bucount;
  int Bscount;
  int Barions;
  // the number of stable charged particles in MinBias and bbbar events
  int ChargedPartMB;
  int ChargedPartBBbar;

};

#endif    // bbbar_Analysis_H_

