#ifndef TTBAR_ANALYSIS_H_ 
#define TTBAR_ANALYSIS_H_ 

#include "baseAnalysis.h"

// forward declarations
namespace HepMC {
  class GenEvent;
}
class TH1D;

/**
                                                                       
 @class ttbarAnalysis.h                                                  
 @brief This class is used for hepmc based validation of ttbar events  

 Init() is used to initialise histograms                               
 Process() is used to process the events and do the analysis and       
                                                                       
 The ouput (histograms) will be saved in a root file                   
                                                                       
 @author Sebastian Johnert, October 2008                               
                                                                       
*/

class ttbarAnalysis: public baseAnalysis
{
public:
  ttbarAnalysis();
  virtual ~ttbarAnalysis();
  
  virtual int Init(double tr_max_eta, double tr_min_pt);
  virtual int Process(HepMC::GenEvent* hepmcevt);

private:
  TH1D *m_evtnr;
  TH1D *m_toppt;
  TH1D *m_toppt_log;
  TH1D *m_topeta;
  TH1D *m_topphi;
  TH1D *m_ptstable;
  TH1D *m_ptstable_log;
  TH1D *m_etastable;
  TH1D *m_phistable;
  TH1D *m_ptstable_charged;
  TH1D *m_ptstable_charged_log;
  TH1D *m_etastable_charged;
  TH1D *m_phistable_charged;
  TH1D *m_pttoppair;
  TH1D *m_pttoppair_log;
  TH1D *m_cmultpart;
  TH1D *m_top_tbar_phi;
  TH1D *m_W_W_phi;
  TH1D *m_W_top_phi;
  TH1D *m_Wpt;
  TH1D *m_Wpt_log;
  TH1D *m_Weta;
  TH1D *m_Wphi;

  TH1D *m_jet_count;
  TH1D *m_jet_pt;
  TH1D *m_jet_pt_log;

  int iSTOP;
};

#endif      // TTBAR_ANALYSIS_H_
 
