// ---------------------------------------------------------------------- 

#include <iostream>

#include "HepMC/GenEvent.h"
#include "HepMC/IO_GenEvent.h"
#include "HepMC/GenParticle.h"
#include "HepMC/GenVertex.h"
#include "HepMC/IO_AsciiParticles.h"
#include "HepMC/SimpleVector.h"
#include "CLHEP/Vector/LorentzVector.h"

#include "TH1.h"
#include "TH2.h"
#include "TFile.h"
#include "TMath.h"
#include "TLorentzVector.h"

#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"
#include "fastjet/JetDefinition.hh"
#include "fastjet/SISConePlugin.hh"

#include "../include/EtMissAnalysis.h"

using namespace std;

// ---------------------------------------------------------------------- 
EtMissAnalysis::EtMissAnalysis()
{}

// ---------------------------------------------------------------------- 
EtMissAnalysis::~EtMissAnalysis()
{}

// ---------------------------------------------------------------------- 
int EtMissAnalysis::Init(double maxEta, double minPt)
{
  m_max_eta = maxEta;  
  m_min_pt = minPt;  
   
  m_outputFileName = "EtMissAnalysis.root";
  m_outputRootDir = "EtMiss";
 
  m_METTruthX = initHist("METTruthX", "METTruthX", "MissingEtTruthX", 100, -100, 100);
  m_METTruthY = initHist("METTruthY", "METTruthY", "MissingEtTruthY", 100, -100, 100);
  m_METTruth = initHist("METTruth", "METTruth", "MissingEtTruth", 100, 0, 100);
  m_METSumTruth = initHist("METSumTruth", "METSumTruth", "MissingSumEtTruth", 100, 0, 100);
  
  return true; 
}

// ---------------------------------------------------------------------- 
int EtMissAnalysis::Process(HepMC::GenEvent* hepmcevt)
{ 
  FindMissingEt(hepmcevt);        // calculate the missing energy (function from baseAnalysis)

  m_METTruthX->Fill(exMissTruth);
  m_METTruthY->Fill(eyMissTruth);
  m_METTruth->Fill(etMissTruth);
  m_METSumTruth->Fill(etsumMissTruth);

  return true;
} 

